@@ -534,7 +534,6 @@ bool gterm_init(struct fb_info **_fbs, size_t *_fbs_count,
             case 180: fb_rotation = FLANTERM_FB_ROTATE_180; break;
             case 270: fb_rotation = FLANTERM_FB_ROTATE_270; break;
         }
-        pmm_free(rotation_str, strlen(rotation_str) + 1);
     }
 
     uint32_t ansi_colours[8];
@@ -666,6 +665,8 @@ bool gterm_init(struct fb_info **_fbs, size_t *_fbs_count,
     char *menu_font_size = config_get_value(config, 0, "TERM_FONT_SIZE");
     if (menu_font_size != NULL) {
         parse_resolution(&tmp_font_width, &tmp_font_height, NULL, menu_font_size);
+        // XXX: Do not allow widths != 8
+        tmp_font_width = 8;
 
         size_t tmp_font_size = (tmp_font_width * tmp_font_height * FLANTERM_FB_FONT_GLYPHS) / 8;
 
@@ -683,6 +684,11 @@ bool gterm_init(struct fb_info **_fbs, size_t *_fbs_count,
         if ((f = uri_open(menu_font)) == NULL) {
             print("menu: Could not open font file.\n");
         } else {
+            if (font_size > f->size) {
+                print("Font size too large for provided font file. Not loading.\n");
+                fclose(f);
+                goto no_load_font;
+            }
             fread(f, font, 0, font_size);
             if (menu_font_size != NULL) {
                 font_width = tmp_font_width;
@@ -764,8 +770,8 @@ no_load_font:;
             fb->framebuffer_height = tmp;
         }
 
-        terms[terms_i] = flanterm_fb_init(ext_mem_alloc,
-                            pmm_free,
+        terms[terms_i] = flanterm_fb_init(ext_mem_alloc_size_t,
+                            pmm_free_size_t,
                             (void *)(uintptr_t)fb->framebuffer_addr,
                             fb->framebuffer_width, fb->framebuffer_height, fb->framebuffer_pitch,
                             fb->red_mask_size, fb->red_mask_shift,
