@@ -35,6 +35,9 @@ EFI_GUID limine_efi_vendor_guid =
 #define TOK_BADKEY 3
 #define TOK_COMMENT 4
 
+static char interface_help_colour[] = "\e[32m";
+static char interface_help_colour_bright[] = "\e[92m";
+
 static char *menu_branding = NULL;
 static char *menu_branding_colour = NULL;
 no_unwind bool booting_from_editor = false;
@@ -234,7 +237,7 @@ refresh:
         print("\n\n");
     }
 
-    print("    \e[32mESC\e[0m Discard and Exit    \e[32mF10\e[0m Boot\n\n");
+    print("    %sESC\e[0m Discard and Exit    %sF10\e[0m Boot\n\n", interface_help_colour, interface_help_colour);
 
     print(serial ? "/" : "┌");
     for (size_t i = 0; i < terms[0]->cols - 2; i++) {
@@ -490,7 +493,7 @@ tab_part:
             if (cursor_offset) {
                 cursor_offset--;
         case GETCHAR_DELETE:
-                for (size_t i = cursor_offset; ; i++) {
+                for (size_t i = cursor_offset; i < buffer_len; i++) {
                     buffer[i] = buffer[i+1];
                     if (!buffer[i])
                         break;
@@ -500,7 +503,12 @@ tab_part:
         case GETCHAR_F10:
             memcpy(saved_orig_entry, buffer, buffer_len);
             saved_orig_entry[buffer_len] = 0;
-            strcpy(saved_title, title);
+            size_t title_len = strlen(title);
+            if (title_len >= sizeof(saved_title)) {
+                title_len = sizeof(saved_title) - 1;
+            }
+            memcpy(saved_title, title, title_len);
+            saved_title[title_len] = 0;
             editor_no_term_reset ? editor_no_term_reset = false : reset_term();
             booting_from_editor = true;
             return buffer;
@@ -555,6 +563,26 @@ static size_t print_tree(size_t offset, size_t window, const char *shift, size_t
         size_t cur_len = 0;
         if (current_entry == NULL)
             break;
+        if (current_entry->sub == NULL) {
+            bool skip_entry = false;
+            char *cur_entry_protocol = config_get_value(current_entry->body, 0, "PROTOCOL");
+            if (cur_entry_protocol) {
+#if defined (UEFI)
+                if (strcmp(cur_entry_protocol, "bios") == 0
+                 || strcmp(cur_entry_protocol, "bios_chainload") == 0) {
+#elif defined (BIOS)
+                if (strcmp(cur_entry_protocol, "efi") == 0
+                 || strcmp(cur_entry_protocol, "uefi") == 0
+                 || strcmp(cur_entry_protocol, "efi_chainload") == 0) {
+#endif
+                    skip_entry = true;
+                }
+                if (skip_entry) {
+                    current_entry = current_entry->next;
+                    continue;
+                }
+            }
+        }
         if (!no_print && base_index + max_entries < offset) {
             goto skip_line;
         }
@@ -649,6 +677,7 @@ static void menu_init_term(void) {
 #if defined (BIOS)
             vga_textmode_init(true);
 #elif defined (UEFI)
+            serial = true;
             term_fallback();
 #endif
         }
@@ -757,6 +786,10 @@ noreturn void _menu(bool first_run) {
         }
     }
 
+#if defined (__riscv)
+    init_riscv(NULL);
+#endif
+
     char *quiet_str = config_get_value(NULL, 0, "QUIET");
     quiet = quiet_str != NULL && strcmp(quiet_str, "yes") == 0;
 
@@ -791,7 +824,23 @@ noreturn void _menu(bool first_run) {
         help_hidden = strcmp(help_hidden_str, "yes") == 0;
     }
 
-    menu_branding = config_get_value(NULL, 0, "INTERFACE_BRANDING");
+    char *interface_help_colour_str = config_get_value(NULL, 0, "INTERFACE_HELP_COLOUR");
+    if (interface_help_colour_str == NULL) {
+        interface_help_colour_str = config_get_value(NULL, 0, "INTERFACE_HELP_COLOR");
+    }
+    if (interface_help_colour_str != NULL) {
+        interface_help_colour[3] = interface_help_colour_str[0];
+        interface_help_colour_bright[3] = interface_help_colour_str[0];
+    }
+
+    {
+        char *tmp = config_get_value(NULL, 0, "INTERFACE_BRANDING");
+        if (tmp != NULL) {
+            size_t len = strlen(tmp) + 1;
+            menu_branding = ext_mem_alloc(len);
+            memcpy(menu_branding, tmp, len);
+        }
+    }
     if (menu_branding == NULL) {
 #if defined (BIOS)
         {
@@ -828,11 +877,18 @@ noreturn void _menu(bool first_run) {
 #endif
     }
 
-    menu_branding_colour = config_get_value(NULL, 0, "INTERFACE_BRANDING_COLOUR");
-    if (menu_branding_colour == NULL)
-        menu_branding_colour = config_get_value(NULL, 0, "INTERFACE_BRANDING_COLOR");
-    if (menu_branding_colour == NULL)
-        menu_branding_colour = "6";
+    {
+        char *tmp = config_get_value(NULL, 0, "INTERFACE_BRANDING_COLOUR");
+        if (tmp == NULL)
+            tmp = config_get_value(NULL, 0, "INTERFACE_BRANDING_COLOR");
+        if (tmp != NULL) {
+            size_t len = strlen(tmp) + 1;
+            menu_branding_colour = ext_mem_alloc(len);
+            memcpy(menu_branding_colour, tmp, len);
+        } else {
+            menu_branding_colour = "6";
+        }
+    }
 
     bool skip_timeout = false;
     struct menu_entry *selected_menu_entry = NULL;
@@ -978,22 +1034,24 @@ refresh:
             set_cursor_pos_helper(0, 3);
             if (max_entries != 0) {
                 if (selected_menu_entry->sub == NULL) {
-                    print("    \e[32mARROWS\e[0m Select    \e[32mENTER\e[0m Boot    %s",
-                          editor_enabled ? "\e[32mE\e[0m Edit" : "");
+                    print("    %sARROWS\e[0m Select    %sENTER\e[0m Boot    %s%s",
+                          interface_help_colour, interface_help_colour, interface_help_colour,
+                          editor_enabled ? "E\e[0m Edit" : "\e[0m");
                 } else {
-                    print("    \e[32mARROWS\e[0m Select    \e[32mENTER\e[0m %s",
+                    print("    %sARROWS\e[0m Select    %sENTER\e[0m %s",
+                          interface_help_colour, interface_help_colour,
                           selected_menu_entry->expanded ? "Collapse" : "Expand");
                 }
             }
 #if defined(UEFI)
             if (reboot_to_firmware_supported) {
                 set_cursor_pos_helper(terms[0]->cols - (editor_enabled ? 37 : 20), 3);
-                print("\e[32mS\e[0m Firmware Setup");
+                print("%sS\e[0m Firmware Setup", interface_help_colour);
             }
 #endif
             if (editor_enabled) {
                 set_cursor_pos_helper(terms[0]->cols - 17, 3);
-                print("\e[32mB\e[0m Blank Entry");
+                print("%sB\e[0m Blank Entry", interface_help_colour);
             }
         }
         set_cursor_pos_helper(x, y);
@@ -1009,7 +1067,8 @@ refresh:
         for (size_t i = timeout; i; i--) {
             set_cursor_pos_helper(0, terms[0]->rows - 1);
             FOR_TERM(TERM->scroll_enabled = false);
-            print("\e[2K\e[32mBooting automatically in \e[92m%u\e[32m, press any key to stop the countdown...\e[0m", i);
+            print("\e[2K%sBooting automatically in %s%u%s, press any key to stop the countdown...\e[0m",
+                  interface_help_colour, interface_help_colour_bright, i, interface_help_colour);
             FOR_TERM(TERM->scroll_enabled = true);
             FOR_TERM(TERM->double_buffer_flush(TERM));
             if ((c = pit_sleep_and_quit_on_keypress(1))) {
@@ -1100,6 +1159,7 @@ timeout_aborted:
 #if defined (BIOS)
                             vga_textmode_init(true);
 #elif defined (UEFI)
+                            serial = true;
                             term_fallback();
 #endif
                         }
@@ -1166,12 +1226,19 @@ noreturn void boot(char *config) {
     init_riscv(config);
 #endif
 
-    char *cmdline = config_get_value(config, 0, "KERNEL_CMDLINE");
-    if (!cmdline) {
-        cmdline = config_get_value(config, 0, "CMDLINE");
-    }
-    if (!cmdline) {
-        cmdline = "";
+    char *cmdline;
+    {
+        char *tmp = config_get_value(config, 0, "KERNEL_CMDLINE");
+        if (!tmp) {
+            tmp = config_get_value(config, 0, "CMDLINE");
+        }
+        if (tmp) {
+            size_t len = strlen(tmp) + 1;
+            cmdline = ext_mem_alloc(len);
+            memcpy(cmdline, tmp, len);
+        } else {
+            cmdline = "";
+        }
     }
 
     char *proto = config_get_value(config, 0, "PROTOCOL");
